--------------------------------------------------------------------------------
--  CURRENCY RELATED FUNCTIONS
--[[
	'bar' = Bar Number
		The dynamic table index of a group's record in: FI_SVPC_DATA["Groups"]
		This is a user interface convention to provide the appearance of sequential bar numbers regardless of the actual group ID.
	
	'gid' = Group ID
		The numeric primary key of a group as found in its data "record" stored at FI_SVPC_DATA.Groups
	
	'bid' = Button ID
		The numeric primary key of a button as found in its data "record" stored at FI_SVPC_DATA.Buttons
	
	'cid' = Currency ID
		The numeric primary key of a "watched" currency as found in its data "record" stored at FI_SVPC_DATA.Currencies
	
	'sid' = Session ID
		The numeric primary key of a farming session as found in its data "record" stored at FI_SVPC_DATA.Sessions
]]--
--------------------------------------------------------------------------------

-- if currency name given, returns matching info if available
function FI_HasTokens( currency )
	-- check if player has any trackable currencies available
	for n=1,GetCurrencyListSize() do
		local name, isHeader, isExpanded, isUnused, isWatched, count, icon, maximum, hasWeeklyLimit, currentWeeklyAmount = GetCurrencyListInfo(n);
		
		if name and (not isHeader) then
			if currency then
				-- query matched, return item data
				if (currency == name) then
					return n, name, count, maximum, hasWeeklyLimit, currentWeeklyAmount;
				end
			else
				-- found a currency, exit loop
				return true;
			end
		end
	end
end

--------------------------------------------------------------------------------
--  CURRENCY FUNCTIONS
--------------------------------------------------------------------------------
function FI_ToggleCurrency( setting, verbose )
	
	if (FI_SVPC_CONFIG.Currency[setting] == nil) then
		FI_Message("Invalid option:  "..setting);
	else
		FI_SVPC_CONFIG.Currency[setting] = FI_Toggle(FI_SVPC_CONFIG.Currency[setting]);
		FI_Currency_Vis();
	end
	
	if verbose then
		FI_Message("Currency "..setting.." = "..strupper(tostring(FI_SVPC_CONFIG.Currency[setting])));
	end
end

function FI_Currency( event )
	if FI_SV_CONFIG.debug then print("[FI_Currency]  Called."); end
	-- VERBOSE DEBUG
	--if FI_SV_CONFIG.debug then print("|cFFFF0000Currencies table:"); table.dump(FI_SVPC_DATA.Currencies); end
	
	-- hud configuration check
	if FI_SVPC_CONFIG.Currency.hud then
		FI_Currency_UpdateHUD(event);
	end
	
	-- tracking configuration check
	if FI_SVPC_CONFIG.Currency.tracking then
		-- one-time startup check
		if (FI_CURRENCY_LOADED == false) then
			FI_DB.check("Currencies");
			FI_CURRENCY_LOADED = true;
		end
		
		FI_Currency_Update(event);
	end
	
	-- apply visual settings
	FI_Currency_Vis();
	
	-- VERBOSE DEBUG
	--if FI_SV_CONFIG.debug then print("|cFFFF0000Currencies table AFTER startup:"); table.dump(FI_SVPC_DATA.Currencies); end
	if FI_SV_CONFIG.debug then print("[FI_Currency]  Finished."); end
end

-- "BackpackTokenFrame" hook
function FI_Currency_OnShow()
	-- configuration check
	if (not FI_SVPC_CONFIG.Currency.tracking) then return; end
	
	if _G["BackpackTokenFrame"]:IsVisible() then
		if FI_SV_CONFIG.debug then print("[FI_Currency_OnShow]  Called."); end
		
		FI_MOVING = true;
			FI_Currency_Update();
			FI_Currency_Vis();
		FI_MOVING = false;
	end
end

-- "TokenFrameContainer" hook
function FI_Currency_OnHide()
	if FI_SV_CONFIG.debug then print("[FI_Currency_OnHide]  Called."); end
	
	FI_MOVING = true;
		FI_Currency_Update();
		FI_Currency_UpdateHUD();
		FI_Currency_Vis();
	FI_MOVING = false;
end

function FI_Click_Currency( self, click, down )
	-- configuration check
	if (not FI_SVPC_CONFIG.Currency.tracking) then return; end
	
	if (click == "RightButton") then
		local f_name = self:GetName(); --BackpackTokenFrameToken1
		local parent = self:GetParent();
		local f_id;
		
		if (parent:GetName() == "BackpackTokenFrame") then
			f_id = tonumber(strsub(f_name, -1));
		else
			f_id = FI_FrameToID(f_name);
		end
		
		local eb_name = "FI_Currency_"..f_id.."_Editbox";

		if FI_SV_CONFIG.debug then print("You CLICKED the "..click.." ("..tostring(down)..") on frame:  "..f_name); end
		
		-- hide existing editboxes
		for cid=1,3 do
			local eb_n = "FI_Currency_"..cid.."_Editbox";
			if _G[eb_n] then _G[eb_n]:Hide(); end
		end
		
		-- create editbox if it doesn't exist yet
		if not _G[eb_name] then
			local s = FI_SVPC_STYLE.anchor;
			
			local f = CreateFrame("EditBox", eb_name, parent, "FI_TPL_Editbox");
			f:SetSize(70,35);
			f:SetJustifyH("CENTER");
			f:SetPoint("TOP", _G["BackpackTokenFrameToken"..f_id], "BOTTOM", 0, 0);
			
			f:SetScript("OnEnterPressed", FI_Currency_SetObjective);
			f:SetScript("OnEscapePressed", f.Hide);
			f:SetScript("OnEditFocusLost", f.Hide);
			
			f:SetMaxBytes(5);
			f:SetNumeric(true);
			f:SetAutoFocus(true);
			f:SetHistoryLines(6);
			
			f.text = f:CreateFontString("label","ARTWORK");
			f.text:SetPoint("LEFT", f, "LEFT", 12, 0);
			f.text:SetFont(s.text.font, s.text.size, s.text.flags);
			f.text:SetVertexColor(s.text.color[1], s.text.color[2], s.text.color[3]);
			f.text:SetAlpha(s.text.alpha);
			--f.text:SetText("Objective:");
		end
		
		-- adjust anchor as needed
		_G[eb_name]:SetPoint("TOP", self, "BOTTOM", 0, 0);
		
		-- get the current info
		local currency = FI_DB.select(FI_SVPC_DATA.Currencies, {id = f_id}, true);
		
		-- update the interface
		if currency then
			_G[eb_name]:SetNumber(currency.objective);
			_G[eb_name]:Show();
			_G[eb_name]:HighlightText();
				
			--FI_Message("Enter a |cFF"..FI_SV_CONFIG.Colors.currency[4]..currency.name.."|r objective at the bottom of your backpack.");
		elseif FI_SV_CONFIG.debug then
			print("[FI_Click_Currency]  DB query failed for 'f_id':  "..f_id); --debug
		end
	end
end

function FI_Click_CurrencyHUD( self, click, down )
	local f_name = self:GetName();
	if FI_SV_CONFIG.debug then print("You CLICKED the "..click.." ("..tostring(down)..") on frame: "..f_name); end
	
	if (click == "LeftButton") then
		f_name = "FI_CurrencyHUD";
		
		if IsShiftKeyDown() then
			print("frame name: "..f_name);
			------------------------------------------------------------
			-- DRAG BAR
			------------------------------------------------------------
			if down then
				_G[f_name]:StartMoving();
			else
				_G[f_name]:StopMovingOrSizing();
			end
		
		else
			------------------------------------------------------------
			-- 
			------------------------------------------------------------
			if down then
				--
			else
				_G[f_name]:StopMovingOrSizing();
			end
			
		end
	
	elseif (click == "RightButton") then
		if IsShiftKeyDown() then
			------------------------------------------------------------
			-- 
			------------------------------------------------------------
			
		else
			------------------------------------------------------------
			-- EDIT OBJECTIVE
			------------------------------------------------------------
			if down then
				--
			else
				FI_Click_Currency(self, click, down);
			end
		end
	end
end

function FI_Currency_Tooltip( cid )
	-- grab currency record for the slot
	local currency = FI_DB.select(FI_SVPC_DATA.Currencies, {id = cid}, true);
	
	if currency.name and (currency.objective > 0) then
		-- set color
		local color;
		if (currency.count < currency.objective) then
			color = FI_SV_CONFIG.Colors.progress[4];
		else
			color = FI_SV_CONFIG.Colors.success[4];
		end
		
		return "Farming Objective: |cFF"..color..currency.objective;
	end
end

function FI_Tooltip_CurrencyHUD( self )
	local cid = FI_FrameToID( self:GetName() );
	local index = FI_HasTokens( GetBackpackCurrencyInfo(cid) );
	
	-- build tooltip
	GameTooltip:SetOwner(self,"ANCHOR_BOTTOMLEFT",50,-30);
	GameTooltip:SetCurrencyToken(index);
	-- display objective
	local line = FI_Currency_Tooltip(cid);
	if line then GameTooltip:AddLine(line);	end
	
	GameTooltip:Show();
end

function FI_Hook_Currency( tooltip )
	if tooltip:GetOwner() then
		local owner = tooltip:GetOwner();
		local parent = owner:GetParent();
		
		if parent and (parent:GetName() == "BackpackTokenFrame") then
			cid = tonumber(strsub(owner:GetName(), -1));
			-- show objective if set
			local line = FI_Currency_Tooltip(cid);
			if line then GameTooltip:AddLine(line);	tooltip:Show(); end
		end
	end
end

--------------------------------------------------------------------------------

function FI_Currency_Update( event )
	if FI_SV_CONFIG.debug then print("[FI_Currency_Update]  Called."); end
	
	-- take a snapshot of current data
	local temp = table.copy(FI_SVPC_DATA.Currencies);
	
	-- process all three backpack "watched currency" slots
	for cid=1,3 do
		-- API query
		local name,count,icon = GetBackpackCurrencyInfo(cid);
		
		local results;
		if name then
			-- server returned something for the current slot
			local query = {
				["name"] = name,
				["count"] = count,
				["icon"] = icon,
			}
			
			-- check for lastcount data in snapshot
			local existing = FI_DB.select(temp, {["name"] = name}, true);
			if existing then
				query["lastcount"] = existing.count;
			else
				query["lastcount"] = count;
			end

			-- check for cached objective data
			if FI_SVPC_CACHE.Currencies then
				local cached = FI_DB.select(FI_SVPC_CACHE.Currencies, {["name"] = name}, true);
				if cached then
					query["objective"] = cached.objective;
				else
					query["objective"] = 0;
					query["success"] = false;
				end
			end
			
			------------------------------------------------------------
			-- save updated record
			------------------------------------------------------------
			results = FI_DB.update(FI_SVPC_DATA.Currencies, {id = cid}, query);
		
		elseif (FI_MOVING == true) and (FI_LOADING == false) then
			------------------------------------------------------------
			-- clear unused "slot"
			------------------------------------------------------------
			results = FI_Currency_Clear(cid);
		
		else
			------------------------------------------------------------
			-- use existing data
			------------------------------------------------------------
			results = FI_DB.select(FI_SVPC_DATA.Currencies, {id = cid}, true);
		end
		
		------------------------------------------------------------
		-- update backpack interface
		------------------------------------------------------------
		local f_name = "FI_Currency_"..cid;
		_G[f_name.."_Objective"]:SetText(results.objective);
		
		------------------------------------------------------------
		-- progress tracking
		------------------------------------------------------------
		if (FI_LOADING == false) then
			FI_Progress(results);
		end
	end
end

function FI_Currency_UpdateHUD()
	-- update Currency HUD
	for cid=1,3 do
		local slot = "FI_CurrencyHUD_"..cid;
		
		-- API query
		local name,count,icon = GetBackpackCurrencyInfo(cid);
		
		if name then
			-- populate slot
			_G[slot.."_Text"]:SetText(count);
			_G[slot.."_Icon"]:SetTexture(icon);
			_G[slot]:Show();
		else
			-- clear slot
			_G[slot.."_Text"]:SetText(0);
			_G[slot.."_Icon"]:SetTexture("");
			_G[slot]:Hide();
		end
	end
end

function FI_Currency_Vis()
	if FI_SV_CONFIG.debug then print("[FI_Currency_Vis]  Called."); end
	
	-- tracking config check
	if FI_SVPC_CONFIG.Currency.tracking then
		-- show backpack objective frames
		for cid=1,3 do
			local currency = FI_DB.select(FI_SVPC_DATA.Currencies, {id = cid}, true);
			
			if currency then
				-- objective frame
				local f_name = "FI_Currency_"..cid;
				-- hud frame
				local hud_slot = "FI_CurrencyHUD_"..cid;
				
				-- make sure things look the way they are supposed to
				if currency.name and (currency.objective > 0) then
					-- set color
					local color;
					if (currency.count < currency.objective) then
						color = FI_SV_CONFIG.Colors.objective;
					else
						color = FI_SV_CONFIG.Colors.success;
					end
					
					_G[f_name.."_Objective"]:SetVertexColor(color[1], color[2], color[3]);
					_G[hud_slot.."_Text"]:SetVertexColor(color[1], color[2], color[3]);
					
					-- set visibility
					_G[f_name]:Show();
				
				else
					-- set color
					color = FI_SV_CONFIG.Colors.objective;
					_G[f_name.."_Objective"]:SetVertexColor(color[1], color[2], color[3]);
					--_G[hud_slot.."_Text"]:SetVertexColor(color[1], color[2], color[3]);
					
					-- set visibility
					_G[f_name]:Hide();

					if FI_SV_CONFIG.debug then print("[FI_Currency_Vis]  Hiding frame "..cid); end
				end
				
				
			elseif FI_SV_CONFIG.debug then
				print("[FI_Currency_Vis]  ERROR! Unable to find currency record for slot "..cid); --debug
			end
		end
	
	else
		-- hide backpack objective frames
		for cid=1,3 do
			local f_name = "FI_Currency_"..cid;
			_G[f_name]:Hide();
		end
	end
	
	-- hud config check
	if (FI_SVPC_CONFIG.Currency.hud == true) then
		-- show currency hud
		_G["FI_CurrencyHUD"]:Show();
	elseif (FI_SVPC_CONFIG.Currency.hud == false) then
		-- hide currency hud
		_G["FI_CurrencyHUD"]:Hide();
	end
end

-- FI_SetObjective wrapper
function FI_Currency_SetObjective( editbox, id, input )
	local cid,amt;
	
	if editbox then
		cid = FI_FrameToID( editbox:GetName() );
		amt = editbox:GetNumber();
		
	elseif id and input then
		cid = tonumber(id);
		amt = tonumber(input);
		
	elseif FI_SV_CONFIG.debug then
		-- error
		print("[FI_Currency_SetObjective]  Input error."); --debug
	end
	
	-- select record
	local currency = FI_DB.select(FI_SVPC_DATA.Currencies, {["id"] = cid}, true);
	
	if currency then
		-- skip updating if input matches current objective
		if (amt ~= currency.objective) then
			-- dew it!
			FI_SetObjective("Currencies", currency.id, amt);
			FI_Currency_Update();
		
		elseif FI_SV_CONFIG.debug then
			print("[FI_Currency_SetObjective]  Duplicate objective input, update skipped."); --debug
		end
			
	elseif FI_SV_CONFIG.debug then
		print("[FI_Currency_SetObjective]  DB query failed for CID:  "..cid); --debug
	end
	
	--hide input frame
	editbox:Hide();
end

-- reset a currency record
function FI_Currency_Clear( cid )
	-- reset data
	local query = {
		["name"] = "",
		["count"] = 0,
		["lastcount"] = 0,
		["objective"] = 0,
		["success"] = false,
	}
	
	local results = FI_DB.update(FI_SVPC_DATA.Currencies, {id = cid}, query);
	
	-- update backpack interface
	local f_name = "FI_Currency_"..cid;
	_G[f_name.."_Objective"]:SetText(0);
	_G[f_name]:Hide();
	
	return results;
end

-- check database structure
function FI_Currency_Check()
	-- validate currency data
	if (GetNumWatchedTokens() > 0) then
		if FI_SV_CONFIG.debug then print("[FI_Currency_Check]  Validating currency records..."); end
		
		local result;
		
		for cid=1,3 do
			local name,count,icon = GetBackpackCurrencyInfo(cid);
			local currency = FI_DB.select(FI_SVPC_DATA.Currencies, {id = cid}, true);
			
			if name then
				if (name == currency.name) then
					result = true;
				else
					result = false; break;
				end
			
			elseif FI_SV_CONFIG.debug then
				print("[FI_Currency_Check]  API query failed on slot "..cid); --debug
				result = false; break;
			end
		end
		
		-- returns false if there is a problem
		return result;
	end
end

--------------------------------------------------------------------------------
--  MONEY TRACKING
--------------------------------------------------------------------------------
function FI_Money( event, arg )
	if FI_SV_CONFIG.debug then FI_Message("FI_Money:  "..event.."  "..arg); end
end
